@@ -3,11 +3,11 @@ package Apache::DBI;
 use DBI ();
 use strict;
 
-#$Id: DBI.pm,v 1.19 1998/02/18 20:16:43 mergl Exp $
+#$Id: DBI.pm,v 1.22 1998/07/26 05:30:53 mergl Exp $
 
 require_version DBI 0.85;
 
-$Apache::DBI::VERSION = '0.78';
+$Apache::DBI::VERSION = '0.80';
 
 $Apache::DBI::DEBUG = 0;
 
@@ -22,7 +22,7 @@ sub connect_on_init { push @ChildConnect, [@_] }
 # when forking a new child the handler establishes all configured connections
 if(Apache->can('push_handlers')) {
     Apache->push_handlers(PerlChildInitHandler => sub {
-       print STDERR "$$ Apache::DBI push_handlers(PerlChildInitHandler) \n" if $Apache::DBI::DEBUG;
+       print STDERR "$$ Apache::DBI PerlChildInitHandler \n" if $Apache::DBI::DEBUG;
        for my $aref (@ChildConnect) {
            shift @$aref;
            DBI->connect(@$aref);
@@ -37,23 +37,42 @@ sub connect {
     unshift @_, $class if ref $class;
     my $drh  = shift;
     my @args = map { defined $_ ? $_ : "" } @_;
-    my $idx  = "$args[0]:$args[1]:$args[2]";
+    my $idx  = "$args[0]~$args[1]~$args[2]";
+
+    # the hash-reference differs between calls even in the same
+    # process, so dereference the hash-reference 
 
-    # the hash_ref differs between calls even in the same process
     my ($key, $val);
     if (3 == $#args && ref $args[3] eq "HASH") {
        while (($key,$val) = each %{$args[3]}) {
-           $idx .= ":$key=$val";
+           $idx .= "~$key=$val";
        }
     } elsif (3 == $#args) {
         pop @args;
     }
 
+    # don't cache connections created during server initialization; they
+    # won't be useful after ChildInit, since multiple processes trying to
+    # work over the same database connection simultaneously will receive
+    # unpredictable query results.
+
+    if ($Apache::ServerStarting) {
+        print STDERR "$$ Apache::DBI skipping connection cache during server startup\n" if $Apache::DBI::DEBUG;
+        return $drh->connect(@args);
+    }
+
+    # check first if there is already a database-handle cached
+    # if this is the case, verifiy the database-handle using the 
+    # ping-method.
+
     if (($Connected{$idx} && $Connected{$idx}->ping)) {
         print STDERR "$$ Apache::DBI already connected to '$idx'\n" if $Apache::DBI::DEBUG;
         return (bless $Connected{$idx}, 'Apache::DBI::db');
     }
 
+    # either there is no database handle-cached or it is not valid,
+    # so get a new database-handle and store it in the chache
+
     $Connected{$idx} = undef;
     $Connected{$idx} = $drh->connect(@args);
     return undef if ! $Connected{$idx};
@@ -77,7 +96,7 @@ Apache::Status->menu_item(
         my($r, $q) = @_;
         my(@s) = qw(<TABLE><TR><TD>Datasource</TD><TD>Username</TD></TR>);
         for (keys %Connected) {
-            push @s, '<TR><TD>', join('</TD><TD>', (split(':', $_))[0,1]), "</TD></TR>\n";
+            push @s, '<TR><TD>', join('</TD><TD>', (split('~', $_))[0,1]), "</TD></TR>\n";
         }
         push @s, '</TABLE>';
         return \@s;
@@ -100,7 +119,7 @@ Apache::DBI - Initiate a persistent database connection
 
  # Configuration in httpd.conf or srm.conf:
 
- PerlModule Apache::DBI  # this comes before all other Apache modules
+ PerlModule Apache::DBI  # this comes before all other ApacheDBI modules
 
 
 =head1 DESCRIPTION
@@ -142,7 +161,7 @@ In a short time this would kill the web server.
 
 Another problem are timeouts: some databases disconnect the client after a 
 certain time of inactivity. The module tries to validate the database handle 
-using the new ping-method of the DBI-module. This method returns true as 
+using the ping-method of the DBI-module. This method returns true as 
 default. If the database handle is not valid and the driver module has no 
 implementation for the ping method, you will get an error when accessing the 
 database. As a work-around you can try to replace the ping method by any 
@@ -152,7 +171,8 @@ This module plugs in a menu item for Apache::Status. The menu lists the
 current database connections. It should be considered incomplete because of 
 the limitations explained above. It shows the current database connections 
 for one specific httpd process, the one which happens to serve the current 
-request. Other httpd children might have other database connections. 
+request. Other httpd children might have other database connections. The 
+Apache::Status module has to be loaded before the Apache::DBI module !
 
 
 =head1 CONFIGURATION
@@ -162,7 +182,7 @@ Add the following line to your httpd.conf or srm.conf:
 
  PerlModule Apache::DBI
 
-It is important, to load this module before any other Apache module ! 
+It is important, to load this module before any other ApacheDBI module ! 
 
 The module provides the additional method:
 
@@ -171,9 +191,21 @@ The module provides the additional method:
 This can be used as a simple way to have apache children establish connections 
 on server startup. This call should be in a startup file (PerlModule, <Perl> 
 or PerlRequire). It will establish a connection when a child is started in 
-that child process. Note that this needs mod_perl-1.07_02 or higher, 
-apache_1.3b3 or higher and that mod_perl needs to be configured with 
-PERL_CHILD_INIT=1 and PERL_STACKED_HANDLERS=1.
+that child process. Note that this needs mod_perl-1.08 or higher, 
+apache_1.3.0 or higher and that mod_perl needs to be configured with 
+
+  PERL_CHILD_INIT=1 PERL_STACKED_HANDLERS=1
+
+For the menu item 'DBI connections' you need to call Apache::Status BEFORE 
+Apache::DBI ! For an example of the configuration order see startup.pl. 
+
+
+=head1 PREREQUISITES
+
+For Apache::DBI you need to enable the appropriate call-back hooks when 
+making mod_perl: 
+
+  perl Makefile.PL DO_HTTPD=1 PERL_CHILD_INIT=1 PERL_STACKED_HANDLERS=1. 
 
 
 =head1 SEE ALSO
